#! /usr/bin/perl


### *** variables ***
###

# Information is stored in variables.
# Variable names start with one of three characters that define 
# the variable type.
# A scalar variable that stores a single value begins with $.
# The following (second) character can be a letter, a digit, 
# or _ (underscore) - if it is a digit, only digits can follow.
# In great majority of cases, using letters is the least confusing and 
# the most descriptive.
# Some examples: $name, $name1, $name_1, $last_name, $LastName.

# Declare and initialize the variable "$name":
#
my $name	= "";

# The statement "my" restricts the variable's scope to the block in which it 
# was defined (here it is the script). It is not required but greatly 
# helps avoid confusion, manage the program and reduce debugging time.

# Good habits:
# - begin your variable names with a letter (following the $ character)
# - make names meaningful so they are easy to understand in larger scripts 
#   (e.g., use "$name" instead of "$a")
# - initialize a new variable - assign a default or null value to it
# - use "my"

### *** input ***
###

# Getting information from the user can be done directly from command prompt:

print "What is your name?\n";
print "Please, type your name (and press enter): ";

$name	= <STDIN>;

# User-typed information, including the newline character, is read from the 
# keyboard (using the <> operator), and assigned to the variable $name. 
# "STDIN" refers to standard input, that is typically the keyboard.

### *** chomp ***
#
# The function "chomp" removes the end of line character(s) from the string 
# entered by the user:
#
chomp $name;

print "Nice to meet you, $name.\n";

# Sometimes it is helpful to use <STDIN> to wait for the user to terminate 
# the program. 
# But be careful - the program will not terminate until the enter key is 
# pressed.
#
print "\n";
print "Exiting. Please, press enter.\n";
<STDIN>;
exit(0);
