#! /usr/bin/perl

use strict;
use warnings;

# A regular expression (regex) corresponds to a pattern than can be 
# applied to a string of characters. It allows for recognition of patterns 
# within strings and for string manipulation.
# There are special characters with special meaning in regular expressions.
# Some of them are:
# .  - any character
# \  - placed before a special character (like the dot) makes it a "regular" 
#      dot (e.g., period in a sentence)
# *  - match 0 or more times
# +  - match 1 or more times
# ?  - match 0 or 1 time
# () - grouping of characters (also stores the matched string within special 
#      variables, such as $1, $2, ...)
# [] - a character that can be any of the ones listed
# {} - range of numbers
# -  - range of characters
# \d - a digit
# \D - not a digit
# \w - a word character, i.e. A, ..., Z, a, ..., z, 0, ..., 9, _
# \W - a non-word character
# \s - a whitespace (including tabulator, end of line and carriage return)
# \S - a non-whitespace
# ^  - placed within [] means the opposite (e.g., [^\S] means \s)
# ^  - placed at the beginning of regex means the beginning of a string
# $  - placed at the end of regex means the end of a string
# /  - usually used as a regex delimiter
# \t - tabulator
# \n - new line
# \b - word boundary
#
# Additionally, modifiers can be used following the closing delimiter, e.g.:
# i - case insensitive
# g - global match


my %hash1	= ();
my $key		= "";
my $var1	= "This is an example and only an example.";
my $backup1	= $var1;
print $var1."\n";


### Step 1
###
print "Step 1\n";
print "\n";

if ($var1 =~ m/is/)
{
	print "Yes\n";
}
else
{
	print "No\n";
}

# The match operator "m" can be omitted if using slashes as delimiters.

<STDIN>;


### Step 2
###
print "Step 2\n";
print "\n";

if ($var1 =~ /^is/)
{
	print "Yes\n";
}
else
{
	print "No\n";
}

<STDIN>;


### Step 3
###
print "Step 3\n";
print "\n";

if ($var1 =~ m/^\w{1,3}is/)
{
	print "Yes\n";
}
else
{
	print "No\n";
}

<STDIN>;


### Step 4
###
print "Step 4\n";
print "\n";

if ($var1 =~ m/^this/i)
{
	print "Yes\n";
}
else
{
	print "No\n";
}

<STDIN>;


### Step 5
###
print "Step 5\n";
print "\n";

while ($var1 =~ m/(\S+)/g)
{
	print "$1_";
}
print "\n";

<STDIN>;


### Step 6
###
print "Step 6\n";
print "\n";

# Let's count the words...
#
while ($var1 =~ m/([^\s\.]+)/g)
{
	if (defined $hash1{$1})
	{
		$hash1{$1}++;		# not the first occurrence of the word stored in $1
	}
	else
	{
		$hash1{$1}	= 1;	# first occurrence of the word stored in $1
	}
}

# ...and let's print them sorted in the descending order 
# according to their count
#
foreach $key (sort {$hash1{$b} <=> $hash1{$a}} keys %hash1)
{
	print $key."\t".$hash1{$key}."\n";
}

print "\n";

<STDIN>;


### Step 7
###
print "Step 7\n";
print "\n";

# Let's replace "an" with "one"
#
$var1	=~ s/an/one/g;
print $var1."\n";

# Oops! We not only have a wrong result but also lost the original copy!
 
<STDIN>;


### Step 8
###
print "Step 8\n";
print "\n";

$var1	= $backup1;
$var1	=~ s/\ban\b/one/g;
print $var1."\n";
 
<STDIN>;


### Step 9
###
print "Step 9\n";
print "\n";

# Alternatively...
#
$var1	= $backup1;
$var1	=~ s/(\s)an(\s)/$1one$2/g;
print $var1."\n";
 

print "\n";
print "Exiting. Please, press enter.\n";
<STDIN>;
exit(0);
